      subroutine mcfm_vegas(myinit,myitmx,myncall,mybin,xinteg,xerr)
************************************************************************
*                                                                      *
*  This routine should perform the sweeps of vegasnr                   *
*                                                                      *
*    Input parameters:                                                 *
*       myinit  :  the vegasnr routine entry point                     *
*       myitmx  :  the number of vegasnr sweeps                        *
*      myncall  :  the number of iterations per sweep                  *
*          bin  :  whether or not the results should be histogrammed   *
*                                                                      *
*    Returned variables:                                               *
*       xinteg  :  value of integration                                *
*         xerr  :  integration error                                   *
*                                                                      *
*    This version is modified to work with photon processes.           *
*    At NLO one can have:                                              *
*                                                                      *
*    virt -> Born + virtual corrections + integrated QCD counterterms  *
*    real -> Real + subtraction terms                                  *  
*                [+ QED photon-quark subtractions if frag=.true.]      *
*    frag -> Fragmentation contributions, incl. integrated QED dipoles *
*    frag -> Fragmentation contributions, incl. integrated QED dipoles *
*                                                                      *
*    tota -> virt + real [frag=.false.]                                *
*    tota -> virt + real + frag [frag=.true.]                          *
*                                                                      *
************************************************************************

      implicit none
      include 'gridinfo.f'
      include 'realwt.f'
      include 'scale.f'
      include 'facscale.f'
      include 'vegas_common.f'
      include 'PDFerrors.f'
      include 'frag.f'
      include 'reset.f'
      include 'masses.f'
      include 'process.f'
      integer myitmx,myncall,myinit,i,j,k,nproc,mynproc
      logical mybin,bin
      double precision sig,sd,chi,sigr,sdr,sigdk,sddk,chidk,
     & sigfrag,sdfrag,chifrag,sigWdk,sdWdk,chiWdk,
     & xreal,xreal2,xinteg,xerr,adjust,myscale,myfacscale,
     & mymb,sumsig,sumsigr,sumsd,sumsdr
      character*4 part,mypart
      common/nproc/nproc
      common/part/part
      common/mypart/mypart
      common/bin/bin
      common/xreal/xreal,xreal2
      double precision lowint,virtint,realint,fragint
      double precision region(2*mxdim),lord_bypart(-1:1,-1:1)
      logical first,myreadin
      common/bypart/lord_bypart
      external lowint,virtint,realint,fragint
      data first/.true./
      save first
           
c--- Initialize all integration results to zero, so that the
c--- total of virt and real may be combined at the end for 'tota'
      sig=0d0
      sigr=0d0
      sigdk=0d0
      sigWdk=0d0
      sigfrag=0d0
      sd=0d0
      sdr=0d0
      sddk=0d0
      sdWdk=0d0
      sdfrag=0d0
      xreal=0d0
      xreal2=0d0
      
      do j=-1,1
      do k=-1,1
        lord_bypart(j,k)=0d0
      enddo
      enddo
      if (PDFerrors) then
        do i=0,maxPDFsets
          PDFxsec(i)=0d0
        enddo
      endif

c--- Controls behaviour of gen_njets: need to reset phase-space
c--- boundaries when going from virt to real (using tota)
c--- need to reset scale also, for special scalestart values
      reset=.false.
      scalereset=.false.

c--- Put the vegasnr parameters in the common block
      itmx=myitmx
      ncall=myncall
      bin=mybin
      
c--- Store value of part in mypart, which will be retained;
c--- also store value of scale in myscale, which will be retained;
c--- part and scale can be changed to make sure that the tota option works.
      mypart=part
      myscale=scale
      myfacscale=facscale
      mynproc=nproc
      mymb=mb

c--- special process: 5FNS + 4FNS: initialization
c---   for process 421 (426) we will actually run
c---   process 411 (416) [5FNS] and then add on the result of process 401 (406) [4FNS]
      if ((mynproc .eq. 421) .or. (mynproc .eq. 426)) then
        nproc=mynproc-10
	sumsig=0d0
	sumsigr=0d0
	sumsd=0d0
	sumsdr=0d0
      endif

   77 continue     
      if ((mynproc .eq. 421) .or. (mynproc .eq. 426)) then
        mb=mymb
	call chooser
        reset=.true.
        scalereset=.true.
c--- special write-out/read-in for 5FNS + 4FNS process
        if (first .eqv. .true.) then
          readin=.false.
          writeout=.true.
	  if (nproc .ge. 411) then
            outgridfile='dvegas_5FNS_'//part//'.grid'
	  else
            outgridfile='dvegas_4FNS_'//part//'.grid'
	  endif 	 
        else
          readin=.true.
          writeout=.false.
	  if (nproc .ge. 411) then
            ingridfile='dvegas_5FNS_'//part//'.grid'
	  else
            ingridfile='dvegas_4FNS_'//part//'.grid'
	  endif 	 
        endif
      endif
      
      
c--- Basic lowest-order integration
      if (part .eq. 'lord') then
       call boundregion(ndim,region)
       call vegasnr(region,ndim,lowint,myinit,myncall,myitmx,
     .               0,sig,sd,chi)
      endif


c---- REMOVE THIS PIECE EVENTUALLY
      if (part .eq. 'frit') then
         ndim=ndim+1
         call boundregion(ndim,region) 
         call vegasnr(region,ndim,lowint,myinit,myncall,myitmx,
     &                 0,sig,sd,chi)
         ndim=ndim-1
      endif
c---- REMOVE THIS PIECE EVENTUALLY

c--- If we're doing the tota integration, then set up the grid info
      if ((mypart .eq. 'tota') .or. (mypart .eq. 'todk')) then        
        if (first .and. (myinit .eq. 1)) then
c-- special input name for virtual grid
            ingridfile='dvegas_virt_'//ingridfile
            myreadin=readin
        else
          if (first .eqv. .true.) then
            readin=.false.
            writeout=.true.
            outgridfile='dvegas_virt.grid'          
          else
            readin=.true.
            writeout=.false.
            ingridfile='dvegas_virt.grid'
          endif
        endif
      endif        

c--- Virtual integration should have one extra dimension
c--- (added and then taken away)
      if (  (mypart .eq. 'virt') .or. (mypart .eq. 'tota')
     . .or. (mypart .eq. 'todk') )  then
        part='virt'        
        reset=.true.
        scalereset=.true.
        ndim=ndim+1
        call boundregion(ndim,region)
        call vegasnr(region,ndim,virtint,myinit,myncall,myitmx,
     .              0,sig,sd,chi)
        ndim=ndim-1
      endif
            
c--- If we're doing the tota integration, then set up the grid info
      if ((mypart .eq. 'tota') .or. (mypart .eq. 'todk')) then
        if (first .and. (myinit .eq. 1)) then
c-- special input name for real grid
          ingridfile(8:11)='real'
          readin=myreadin
        else
          if (first .eqv. .true.) then
            readin=.false.
            writeout=.true.
            outgridfile='dvegas_real.grid'          
          else
            readin=.true.
            writeout=.false.
            ingridfile='dvegas_real.grid'
          endif
        endif        
      endif 

c--- Real integration should have three extra dimensions
c--- 'realwt' is a special option that in general should be false
c--- ('realwt' true samples the integral according to the
c---   unsubtracted real emission weight)
      if (mypart .eq. 'real') then
        part='real'
        scalereset=.true.
        if (realwt) then
          nprn=0
        endif
        xreal=0d0
        xreal2=0d0
        ndim=ndim+3
        call boundregion(ndim,region)
        call vegasnr(region,ndim,realint,myinit,myncall,myitmx,
     .              0,sigr,sdr,chi)
        ndim=ndim-3
        write(6,*) 
        ncall=myncall
        if (realwt) then
          sigr=xreal
          sdr=dsqrt(abs((xreal2-xreal**2)/dfloat(ncall)))
          write(6,*) itmx,' iterations of ',ncall,' calls'
          write(6,*) 'Value of subtracted integral',sigr
          write(6,*) 'Error on subtracted integral',sdr
        endif
      endif
      if ((mypart .eq. 'tota') .or. (mypart .eq. 'todk')) then
        scale=myscale
        facscale=myfacscale
        part='real'
        reset=.true.
        if (realwt) then
          nprn=0
        endif
        xreal=0d0
        xreal2=0d0
        adjust=(dfloat(ndim+3))/(dfloat(ndim+1))
        ncall=int(dfloat(myncall)**adjust)/2
        write(6,*) 'Adjusting number of points for real to',ncall
        ndim=ndim+3
        call boundregion(ndim,region)
        call vegasnr(region,ndim,realint,myinit,ncall,myitmx,
     .              0,sigr,sdr,chi)
        ndim=ndim-3
        write(6,*) 
        ncall=myncall

        if (realwt) then
          sigr=xreal
          sdr=dsqrt(abs((xreal2-xreal**2)/dfloat(ncall)))
          write(6,*) itmx,' iterations of ',ncall,' calls'
          write(6,*) 'Value of subtracted integral',sigr
          write(6,*) 'Error on subtracted integral',sdr
        endif
      endif      

c--- If we're doing the todk integration, then set up the grid info
      if (mypart .eq. 'todk') then
        if (first .and. (myinit .eq. 1)) then
c-- special input name for real grid
          ingridfile(8:11)='redk'
          readin=myreadin
        else
          if (first .eqv. .true.) then
            readin=.false.
            writeout=.true.
            outgridfile='dvegas_redk.grid'          
          else
            readin=.true.
            writeout=.false.
            ingridfile='dvegas_redk.grid'
          endif
        endif        
      endif 
      
      if (mypart .eq. 'todk')  then
        scale=myscale
        nproc=nproc+1
        call chooser
        part='real'
        reset=.true.
        if (realwt) then
          nprn=0
        endif
        xreal=0d0
        xreal2=0d0
        adjust=(dfloat(ndim+3))/(dfloat(ndim+1))
        ncall=int(dfloat(myncall)**adjust)/2
        write(6,*) 'Adjusting number of points for real to',ncall
        ndim=ndim+3
        call boundregion(ndim,region)
        call vegasnr(region,ndim,realint,myinit,ncall,myitmx,
     .              0,sigdk,sddk,chidk)
        ndim=ndim-3
        write(6,*) 
        ncall=myncall
        nproc=nproc-1
        call chooser

        if (realwt) then
          sigdk=xreal
          sddk=dsqrt(abs((xreal2-xreal**2)/dfloat(ncall)))
          write(6,*) itmx,' iterations of ',ncall,' calls'
          write(6,*) 'Value of subtracted integral',sigdk
          write(6,*) 'Error on subtracted integral',sddk
        endif
      endif      

c--- If we're doing the todk integration for ttbar production with
c--- a hadronic W decay, then set up the grid info
      if ((mypart .eq. 'todk') .and. (case .eq. 'tt_bbh')) then
        if (first .and. (myinit .eq. 1)) then
c-- special input name for real grid
          ingridfile(8:11)='rWdk'
          readin=myreadin
        else
          if (first .eqv. .true.) then
            readin=.false.
            writeout=.true.
            outgridfile='dvegas_rWdk.grid'          
          else
            readin=.true.
            writeout=.false.
            ingridfile='dvegas_rWdk.grid'
          endif
        endif        
      endif 
      
      if ((mypart .eq. 'todk') .and. (case .eq. 'tt_bbh')) then
        scale=myscale
        nproc=nproc+2
        call chooser
        part='real'
        reset=.true.
        if (realwt) then
          nprn=0
        endif
        xreal=0d0
        xreal2=0d0
        adjust=(dfloat(ndim+3))/(dfloat(ndim+1))
        ncall=int(dfloat(myncall)**adjust)/2
        write(6,*) 'Adjusting number of points for real to',ncall
        ndim=ndim+3
        call boundregion(ndim,region)
        call vegasnr(region,ndim,realint,myinit,ncall,myitmx,
     .              0,sigWdk,sdWdk,chiWdk)
        ndim=ndim-3
        write(6,*) 
        ncall=myncall
        nproc=nproc-2
        call chooser

        if (realwt) then
          sigWdk=xreal
          sdWdk=dsqrt(abs((xreal2-xreal**2)/dfloat(ncall)))
          write(6,*) itmx,' iterations of ',ncall,' calls'
          write(6,*) 'Value of subtracted integral',sigdk
          write(6,*) 'Error on subtracted integral',sddk
        endif
      endif      

c--- Fragmentation contribution: only if frag is set to .true.
c--- If we're doing the tota integration and , then set up the grid info
      if ((mypart .eq. 'tota') .and. (frag)) then
        if (first .and. (myinit .eq. 1)) then
c-- special input name for real grid
          ingridfile(8:11)='frag'
          readin=myreadin
        else
          if (first .eqv. .true.) then
            readin=.false.
            writeout=.true.
            outgridfile='dvegas_frag.grid'          
          else
            readin=.true.
            writeout=.false.
            ingridfile='dvegas_frag.grid'
          endif
        endif        
      endif 
      
      if ( ((mypart .eq. 'tota') .and. (frag))
     & .or. (mypart .eq. 'frag') ) then
         part='frag'
         scale=myscale
         facscale=myfacscale
         if (mypart .eq. 'frag') then
	   scalereset=.true.
         else
	   reset=.true.
         endif
	 rescale=.true.		! turn on rescaling for this piece
         ncall=myncall
c         write(6,*) 'Adjusting number of points for frag to',ncall
         ndim=ndim+1
         call boundregion(ndim,region) 
         call vegasnr(region,ndim,fragint,myinit,myncall,myitmx,
     &                 0,sigfrag,sdfrag,chifrag)
         ndim=ndim-1
	 rescale=.false. 	! turn rescaling off again
      endif

c--- special process: 5FNS + 4FNS: initialization
c---   for process 421 (426) we will actually run
c---   process 411 (416) [5FNS] and then add on the result of process 401 (406) [4FNS]
      if ((mynproc .eq. 421) .or. (mynproc .eq. 426)) then 
        sumsig=sumsig+sig
        sumsigr=sumsigr+sigr
        sumsd=sumsd+sd**2
        sumsdr=sumsdr+sdr**2
        if (nproc .ge. 411) then
          nproc=nproc-10
	  goto 77
	endif
c--- return nproc to the value from the input file
        nproc=mynproc	 
	sig=sumsig
	sigr=sumsigr
	sd=dsqrt(sumsd)
	sdr=dsqrt(sumsdr)
      endif
      
c--- calculate integration variables to be returned
      xinteg=sig+sigr+sigdk+sigWdk+sigfrag
      xerr=dsqrt(sd**2+sdr**2+sddk**2+sdWdk**2+sdfrag**2)      
      
c--- return part and scale to their real values
      part=mypart
      scale=myscale
      first=.false.
      
      return
      end
      
      
      subroutine boundregion(idim,region)
c--- Initializes integration region [0,1] for each variable
c--- in the idim-dimensional integration range
      implicit none
      include 'mxdim.f'
      integer i,idim
      double precision region(2*mxdim)
      
      do i=1,idim
      region(i)=0d0
      region(i+idim)=1d0
      enddo
      
      return
      end
      
      
